#include "global.h"
#ifndef __DB_HEADER__
#define __DB_HEADER__


#define MAX_USERLEVEL 3

///////////////////////////////////////////////////////

struct structColor {
	uint8 red;  
	uint8 green;
	uint8 blue;
};


struct structWeapon {
	char name[32];
	float heading;		// direction of fire relative to ship
	float coverage;	// angle of coverage that weapon can fire in
	uint32 power;		// max power required to use
	uint32 damage;		// max damage at full power
	int x;
	int y;
	struct structWeapon *next;
	struct structWeapon *prev;
};
struct structSystem {
	uint32 type;
	// not complete
};
struct structXPoint {
	int r;  
	float a;
};
struct structPolygon {   
	uint32 nr_points;
	struct structColor color;
	struct structXPoint points[1];
};
struct structShipClass {
	uint32 id;
	char name[32];

	uint32 flags;
	uint32 max_armor;   
	uint32 max_sheild;
	uint32 sensor_range;
	uint32 mass;
	uint32 max_fuel;
	int max_fore_velocity;
	int max_aft_velocity;
	float turn_rate;                // radians per second

	struct structShipClass *next;
	struct structShipClass *prev;

	unsigned nr_weapons;
	struct structWeapon *weapons;

	uint32 nr_polygons;
	struct structPolygon *polygons[1];
};


struct structUser {
	unsigned id;
	char username[32];
	int level;
	int newlevel;
	struct structColor color1;
	struct structColor color2;
	struct structUser *next;
	struct structUser *prev;
};
struct structAllies {
	uint32 allies[1];
};


struct structMapClass {
	uint32 id;
	char name[32];			// star, black hole, nebula, planet, asteroid, wormhole
	struct structMapClass *next;
	struct structMapClass *prev;
	unsigned nr_polygons;
	struct structPolygon *polygons[1];	// realloc'd to hold more than one polygon
};


struct structMapObject {
	uint32 id; 
	char name[32];
	uint32 class_id;
	struct structColor color1;
	struct structColor color2;
	uint32 quantity_metal;  uint32 server_id;  
	uint32 quantity_gas;    uint32 destination_id;  
	// server_id is used for wormholes and is the server_id of the destination point
	// destination_id is the mapobject id of the other wormhole
	int gravity;
	struct structMapObject *next;
	struct structMapObject *prev;
};


struct structObject {
	uint32 id;
	char name[32];
	uint32 class_id;
	uint32 user_id;
	int velocity;
	float heading;
	unsigned armor[16];
	unsigned sheilds;
	unsigned fuel;
	unsigned sections;

	uint32 flags;
	uint32 x;
	uint32 y;
	struct structObject *next;
	struct structObject *prev;
};


struct structScript {
	struct structScript *next;
	struct structScript *prev;
	unsigned length;
	char code[1];			// realloc'd to hold more than one character
};



extern struct structShipClass *shipclass_db;
extern struct structUser *user_db;
extern struct structAllies *allies_db;
extern struct structObject *object_db;
extern struct structMapClass *mapclass_db;
extern struct structMapObject *mapobject_db;
extern struct structScript *script_db;

extern uint32 nr_shipclass, nr_users, nr_allies, nr_objects, nr_mapclass, nr_mapobjects, nr_scripts;

extern char db_path[100];

///////////////////////////////////////////////////////

struct structUser	*userFindByName(char *);
struct structUser	*userFindByID(uint32);
void			userUpdateLevels();
void			userChangeLevels();

struct structShipClass	*shipclassFindByID(uint32);

void			ssscanf(char **, char *, void *);


#endif
