#include "gl.h"

///////////////////////////////////////////////////////

static int config[] = {GLX_RGBA,GLX_DOUBLEBUFFER,GLX_DEPTH_SIZE,1,None};

///////////////////////////////////////////////////////

int glx_init(FL_OBJECT *ob){
       XVisual *vi;
       GLXContext context;

       /* query for OpenGL capabilities */

       if(!glxQueryExtension(fl_display, 0, 0))
       {
          fprintf(stderr,"OpenGL is not supported\n");
          return -1;    /* signal the caller we have failed */
       }

       /* select the desired visual */

       if(!(vi = glxChooseVisual(fl_display, fl_screen, config)))
          return -1;

       /* change canvas visual/colormap based on what we've got */
       fl_set_canvas_visual(ob, vi->visual);
       fl_set_canvas_depth(ob, vi->depth);

       /* we need to create a colormap appropriate for the visual we get.
        * Also it is a good idea to fill it with xform's default
        * colors to reduce flashing in case the canvas visual is not
        * the same as the visual rest of the form is using
        */
       fl_set_canvas_colormap(ob, fl_create_colormap(vi, 1));

       if(!(context = glxCreateContext(fl_display, vi, None, GL_TRUE)))
       {
          fprintf(stderr,"Can't create GLX Context\n");
          return -1;  
       }

       /* use the c_vdata field to store this context. Similar to
        * u_vdata, the main parts of the library does not reference or
        * modify c_vdata.
        */
       ob->c_vdata = context;
       return 0;
}


// Routine activate and cleanup can be coded in a similar fashion 
int glx_activate(FL_OBJECT * ob){
       glXMakeCurrent(fl_display, FL_ObjWin(ob), ob->c_vdata);
       return 0;
}


int glx_cleanup(FL_OBJECT * ob){
      if(ob->c_vdata)
          glXDestroyContext(fl_display, ob->c_vdata);
      ob->c_vdata = 0;
      return 0;
}


// With the above routines in place, we write the glcanvas interface routine just like the interface
// routine for any other objects 
FL_OBJECT *fl_create_glcanvas(int type, FL_Coord x, FL_Coord y, 
				FL_Coord w, FL_Coord h, const char *label){
      FL_OBJECT *ob = fl_create_canvas(type, x, y, w, h, label);
      fl_modify_canvas_prop(ob, glx_init, glx_activate, glx_cleanup);
      return ob;
}


FL_OBJECT *fl_add_glcanvas(int type, FL_Coord x, FL_Coord y,
				FL_Coord w, FL_Coord h, const char *label){
      FL_OBJECT *ob = fl_create_glcanvas(type, x, y, w, h, label);
      fl_add_object(fl_current_form, ob);
      return ob;
}
