#include "db.h"

#define colorRGB(A,X) A(tmp->color##X.red), A(tmp->color##X.green), A(tmp->color##X.blue)

//////////////////////////////////////////////////////

struct structShipClass *shipclass_db;
struct structUser *user_db;
struct structObject *object_db[16];
struct structMapClass *mapclass_db;
struct structMapObject *mapobject_db[16];
struct structScript *script_db;			// global scripts

uint32 nr_shipclass, nr_users, nr_objects[16], nr_mapclass, nr_mapobjects[16], nr_scripts;

uint32 max_user_id, max_object_id, total_objects, total_mapobjects;

char db_path[100];

//////////////////////////////////////////////////////

void userLoadAll(){
	FILE *dbfile;
	char filename[120];
	struct structUser *tmp;
	int x;

	strcpy(filename, db_path);
	strcat(filename, "user.db");
	if((dbfile=fopen(filename, "r")) == NULL){
		printf("Unable to find user.db file, creating new file.\n");
		user_db = NULL;
		nr_users = 0;
		max_user_id = 1;
	} else {
		fscanf(dbfile, "users=%d maxuid=%d\n", &nr_users, &max_user_id);
		user_db = NULL;
		for(x=0; x<nr_users; x++){
			tmp = (struct structUser *)malloc(sizeof(struct structUser));
			fscanf(dbfile, "=%u =%s =%s ", &(tmp->id), tmp->username, tmp->password);
			fscanf(dbfile, "=%u %u %u ", colorRGB(&,1));
			fscanf(dbfile, "=%u %u %u\n", colorRGB(&,2));
			if(user_db==NULL){
				tmp->next = tmp;
				tmp->prev = tmp;
			} else {
				tmp->next = user_db;
				tmp->prev = user_db->prev;
				user_db->prev->next = tmp;
				user_db->prev = tmp;
			}
			user_db = tmp;
		}
		fclose(dbfile);
		printf(":\tloaded %d user(s)\n", nr_users);
	}
}


void userSaveAll(){
	FILE *dbfile;
	char filename[120];
	struct structUser *tmp;
	int x;

	strcpy(filename, db_path);
	strcat(filename, "user.db");
	if((dbfile=fopen(filename, "w")) == NULL){
		printf("Unable to open user.db for writing, not saved.\n");
		return;
	}
	fprintf(dbfile, "users=%u maxuid=%u\n", nr_users, max_user_id);
	tmp = user_db;
	for(x=0; x<nr_users; x++){
		fprintf(dbfile, "=%u =%s =%s ", tmp->id, tmp->username, tmp->password);
		fprintf(dbfile, "=%u %u %u ", colorRGB(,1));
		fprintf(dbfile, "=%u %u %u\n", colorRGB(,2));
		tmp = tmp->next;
	}
	fclose(dbfile);
	return;
}


uint32 userValidate(char *uname, char *passwd){
	int x;
	struct structUser *tmp;

	tmp = user_db;
	for(x=0; x<nr_users; x++){
		if(!strcmp(tmp->username, uname)){
			if(!strcmp(tmp->password, passwd)) return(tmp->id);
			return(0);
		}
		tmp = tmp->next;
	}
	return(0);
}


uint32 userAdd(char *uname, char *passwd){
	int x;
	struct structUser *tmp;

	tmp = user_db;
	for(x=0; x<nr_users; x++){
		if(!strcmp(tmp->username, uname)) return(0);
		tmp = tmp->next;
	}
	max_user_id++;
	nr_users++;
	tmp = (struct structUser *)malloc(sizeof(struct structUser));
	tmp->id = max_user_id;
	strcpy(tmp->username, uname);
	strcpy(tmp->password, passwd);
	tmp->color1.red=1; tmp->color1.green=2; tmp->color1.blue=3;
	tmp->color2.red=4; tmp->color2.green=5; tmp->color2.blue=6;
	if(user_db==NULL){
		tmp->next = tmp;
		tmp->prev = tmp;
	} else {
		tmp->next = user_db;
		tmp->prev = user_db->prev;
		user_db->prev->next = tmp;
		user_db->prev = tmp;
	}
	user_db = tmp;
	userSaveAll();
	return(tmp->id);
}


void userSendAll(int s, uint32 user_id, int resumed){
	int x, level;
	struct structUser *tmp, *new;
	struct client_connection *c;
	char data[BUFFER_SIZE];

	new = NULL;
	tmp = user_db;
	for(x=0; x<nr_users; x++){

		// get level
		level = 0;

		// find user entry for uid receiving user DB
		if(tmp->id == user_id) new = tmp;

		// send each user DB entry to client
		sprintf(data, "%c%u %s %u %u %u %u %u %u %u", (char)CMD__NEWUSER, tmp->id, tmp->username,
			level, tmp->color1.red, tmp->color1.blue, tmp->color1.green,
			tmp->color2.red, tmp->color2.blue, tmp->color2.green);
		clientSend(s, data);
		tmp = tmp->next;
	}

	// if client is new then notify each connected client of the new user
	if(!resumed && (new != NULL)){
		c = clients;
		sprintf(data, "%c%d %s 0", (char)CMD__NEWUSER, new->id, new->username);
		for(x=0; x<nr_clients; x++){
			if(c->uid != new->id) clientSend(c->s, data);
			c = c->next;
		}
	}
}


void userStoreNote(unsigned uid, char *note){
	printf("UserDB: notes not implimented yet\n");
}


//////////////////////////////////////////////////////


void mapLoad(){
}
void mapSave(){
}
void mapSendAll(int s){
}


//////////////////////////////////////////////////////


void scriptLoadAll(){
}


//////////////////////////////////////////////////////


void objectLoadAll(){
	FILE *dbfile;
	char filename[120];
	struct structObject *tmp;
	int x, sector, count;

	strcpy(filename, db_path);
	strcat(filename, "object.db");
	for(x=0; x<16; x++) object_db[x] = NULL;
	total_objects = 0;

	if((dbfile=fopen(filename, "r")) == NULL){
		printf("Unable to find object.db file, creating new file.\n");
		nr_users = 0;
		max_object_id = 1;
	} else {
		fscanf(dbfile, "objects=%u %u %u %u %u %u %u %u %u %u %u %u %u %u %u %u maxoid=%u\n",
			&nr_objects[0], &nr_objects[1], &nr_objects[2], &nr_objects[3], &nr_objects[4], 
			&nr_objects[5], &nr_objects[6], &nr_objects[7], &nr_objects[8], &nr_objects[9],
			&nr_objects[10], &nr_objects[11], &nr_objects[12], &nr_objects[13], &nr_objects[14],
			&nr_objects[15], &max_object_id);
		for(sector=0; sector<16; sector++){
			count = nr_objects[sector];
			for(; count>0; count--){
				tmp = (struct structObject *)malloc(sizeof(struct structObject));
				fscanf(dbfile, "%u %s %u %u %d %f", &(tmp->id), tmp->name, &(tmp->class_id),
					&(tmp->user_id), &(tmp->velocity), &(tmp->heading));
				for(x=0; x<16; x++) fscanf(dbfile, "%u", &(tmp->armor[x]));
				fscanf(dbfile, "%u %u %u %u %u %u %u %u\n", &(tmp->sheilds), &(tmp->fuel),
					&(tmp->sections), &(tmp->flags), &(tmp->x), &(tmp->y), &(tmp->distance),
					&(tmp->nr_events));

// load in each event entry
// load in thread status
// load in order

				if(object_db[sector]==NULL){
					tmp->next = tmp;
					tmp->prev = tmp;
				} else {
					tmp->next = object_db[sector];
					tmp->prev = object_db[sector]->prev;
					object_db[sector]->prev->next = tmp;
					object_db[sector]->prev = tmp;
				}
				object_db[sector] = tmp;
				total_objects++;
			}
		}
		fclose(dbfile);
		printf(":\tloaded %d object(s)\n", total_objects);
	}
}


void objectSaveAll(){
/*
	FILE *dbfile;
	char filename[120];
	struct structUser *tmp;
	int x;

	strcpy(filename, db_path);
	strcat(filename, "user.db");
	if((dbfile=fopen(filename, "w")) == NULL){
		printf("Unable to open user.db for writing, not saved.\n");
		return;
	}
	fprintf(dbfile, "users=%d maxuid=%d\n", nr_users, max_uid);
	tmp = user_db;
	for(x=0; x<nr_users; x++){
		fprintf(dbfile, "=%u =%s =%s ", tmp->uid, tmp->username, tmp->password);
		fprintf(dbfile, "=%d %d %d ", colorRGB(,1));
		fprintf(dbfile, "=%d %d %d ", colorRGB(,2));
		fprintf(dbfile, "=%d %d %d\n", colorRGB(,3));
		tmp = tmp->next;
	}
	fclose(dbfile);
	return;
*/
}


struct structObject *objectCreate(uint32 class){
//	int x;
	struct structObject *tmp;

	tmp = object_db[0];
/*
	for(x=0; x<nr_users; x++){
		if(!strcmp(tmp->username, uname)) return(0);
		tmp = tmp->next;
	}
	max_uid++;
	nr_users++;
	tmp = (struct user_info *)malloc(sizeof(struct user_info));
	tmp->uid = max_uid;
	strcpy(tmp->username, uname);
	strcpy(tmp->password, passwd);
	tmp->color1.red=1; tmp->color1.green=2; tmp->color1.blue=3;
	tmp->color2.red=4; tmp->color2.green=5; tmp->color2.blue=6;
	tmp->color3.red=7; tmp->color3.green=8; tmp->color3.blue=9;
	if(user_db==NULL){
		tmp->next = tmp;
		tmp->prev = tmp;
	} else {
		tmp->next = user_db;
		tmp->prev = user_db->prev;
		user_db->prev->next = tmp;
		user_db->prev = tmp;
	}
	user_db = tmp;
	saveUserDB();
*/
	return(tmp);
}


void objectDestroy(struct structObject *object){
}


void objectSendAll(int s, uint32 user_id){
	int a, b, sector;
	struct structObject *tmp;
	char data[BUFFER_SIZE], string[20];

	for(sector=0; sector<16; sector++){
	tmp = object_db[sector];
	for(a=0; a<nr_objects[sector]; a++){
		if(tmp->user_id == user_id){
			sprintf(data, "%c%u %s %u %u %d %f %u %u %u %u %u %u", (char)CMD__OBJECT, tmp->id,
				tmp->name, tmp->class_id, user_id, tmp->velocity, tmp->heading, tmp->sheilds,
				tmp->fuel, tmp->sections, tmp->flags, tmp->x, tmp->y);
			for(b=0; b<16; b++){
				sprintf(string, " %u", tmp->armor[b]);
				strcat(data, string);
			}
			clientSend(s, data);
		}
		tmp = tmp->next;
	}}
}


//////////////////////////////////////////////////////


void shipclassLoadAll(){
	FILE *dbfile;
	char filename[120];
	struct structShipClass *tmp;
	struct structPolygon *tmp2;
	struct structWeapon *tmp3;
	int a, b, c, x, y;
	uint32 size, size2;

	strcpy(filename, db_path);
	strcat(filename, "shipclass.db");
	shipclass_db = NULL;
	nr_shipclass = 0;

	if((dbfile=fopen(filename, "r")) == NULL){
		printf("Unable to find shipclass.db file.\n");
	} else {
		fscanf(dbfile, "classes=%u\n", &nr_shipclass);
		for(a=0; a<nr_shipclass; a++){
			size = sizeof(struct structShipClass);
			tmp = (struct structShipClass *)malloc(size);
			fscanf(dbfile, "%u %s %u %u %u %u %u %u %d %d %f %u %u\n", &(tmp->id), tmp->name,
				&(tmp->flags), &(tmp->max_armor), &(tmp->max_sheild),
				&(tmp->sensor_range), &(tmp->mass), &(tmp->max_fuel),
				&(tmp->max_fore_velocity), &(tmp->max_aft_velocity),
				&(tmp->turn_rate), &(tmp->nr_weapons), &(tmp->nr_polygons));
			for(b=0; b<(tmp->nr_polygons); b++){
				if(b>0){
					size += sizeof(struct structPolygon *);
					tmp = (struct structShipClass *)realloc(tmp, size);
				}
				size2 = sizeof(struct structPolygon);
				tmp2 = (struct structPolygon *)malloc(size2);
				fscanf(dbfile, "\t%u ", &(tmp2->nr_points));
				for(c=0; c<(tmp2->nr_points); c++){
					if(c>0){
						size2 += sizeof(struct structXPoint);
						tmp2 = (struct structPolygon *)realloc(tmp2, size2);
					}
	///////////////////////////////////////////////////////////////////////////
	//				fscanf(dbfile, " (%d %f)", &(tmp2->points[c].r),
	//					&(tmp2->points[c].a));
	///////////////////////////////////////////////////////////////////////////
					// convert x,y to r,a until the shape builder program is done
					// its easier to define shapes with x,y pairs

					fscanf(dbfile, " (%d %d)", &x, &y);
					tmp2->points[c].r = sqrt(pow(x,2) + pow(y,2));
					if(x == 0){
						if(y > 0) tmp2->points[c].a = 1.57079632680;
						else tmp2->points[c].a = -1.57079632680;
					} else {
						tmp2->points[c].a = atan(((float)y)/((float)x));
						if(x < 0){
							if(y > 0) tmp2->points[c].a = 3.1415926 + tmp2->points[c].a;
							else tmp2->points[c].a += 3.1415926;
						}
					}
	///////////////////////////////////////////////////////////////////////////
				}
				tmp->polygons[b] = tmp2;
				fscanf(dbfile, "\n");
			}
			for(b=0; b<(tmp->nr_weapons); b++){
				tmp3 = (struct structWeapon *)malloc(sizeof(struct structWeapon));
				fscanf(dbfile, "\t%s %f %f %u %u %d %d\n", tmp3->name, &(tmp3->heading),
					&(tmp3->coverage), &(tmp3->power), &(tmp3->damage),
					&(tmp3->x), &(tmp3->y));
				if((tmp->weapons) == NULL){
					tmp3->next = tmp3;
					tmp3->prev = tmp3;
				} else {
					tmp3->next = tmp->weapons;
					tmp3->prev = tmp->weapons->prev;
					tmp->weapons->prev->next = tmp3;
					tmp->weapons->prev = tmp3;
				}
				tmp->weapons = tmp3;
			}

			if(shipclass_db == NULL){
				tmp->next = tmp;
				tmp->prev = tmp;
			} else {
				tmp->next = shipclass_db;
				tmp->prev = shipclass_db->prev;
				shipclass_db->prev->next = tmp;
				shipclass_db->prev = tmp;
			}
			shipclass_db = tmp;
		}
		fclose(dbfile);
		printf(":\tloaded %d shipclass(es)\n", nr_shipclass);
	}
}


void shipclassSendAll(int s){
	int a, b, c;
	struct structShipClass *tmp;
	struct structWeapon *tmp2;
	char data[BUFFER_SIZE], string[50];

	tmp = shipclass_db;
	for(a=0; a<nr_shipclass; a++){
		sprintf(data, "%c%u %s %u %u %u %u %u %u %d %d %f %u %u", (char)CMD__CLASS2D, tmp->id,
			tmp->name, tmp->flags, tmp->max_armor, tmp->max_sheild, tmp->sensor_range,
			tmp->mass, tmp->max_fuel, tmp->max_fore_velocity, tmp->max_aft_velocity,
			tmp->turn_rate, tmp->nr_weapons, tmp->nr_polygons);
		for(b=0; b<(tmp->nr_polygons); b++){
			sprintf(string, " %u", tmp->polygons[b]->nr_points);
			strcat(data, string);
			for(c=0; c<tmp->polygons[b]->nr_points; c++){
				sprintf(string, " %d %f", tmp->polygons[b]->points[c].r,
					tmp->polygons[b]->points[c].a);
				strcat(data, string);
			}
		}
		tmp2 = tmp->weapons;
		for(b=0; b<(tmp->nr_weapons); b++){
			sprintf(string, " %s %f %f %u %u %d %d", tmp2->name, tmp2->heading, tmp2->coverage,
				tmp2->power, tmp2->damage, tmp2->x, tmp2->y);
			strcat(data, string);
		}
		clientSend(s, data);
		tmp = tmp->next;
	}
}
