#include "global.h"
#ifndef __DB_HEADER__
#define __DB_HEADER__

////////////////////////////////////////////////

struct structColor {
	uint8 red;  
	uint8 green;
	uint8 blue;
};


struct structWeapon {
	char name[32];
	float heading;		// direction of fire relative to ship
	float coverage;		// angle of coverage that weapon can fire in, half of total coverage
	uint32 power;		// max power required to use
	uint32 damage;		// max damage at full power
//	uint32 torps/mines/probes;		// if launcher then this is number of torps
// incomplete
	int x;
	int y;
	struct structWeapon *next;
	struct structWeapon *prev;
};
struct structSystem {
	uint32 type;
	// not complete
};
struct structXPoint {
	int r;
	float a;
};
struct structPolygon {
	uint32 nr_points;
	struct structColor color;
	struct structXPoint points[1];
};
struct structShipClass {
	uint32 id;
	char name[32];

	uint32 flags;			// bit0 = regen armor
	uint32 max_armor;
	uint32 max_sheild;
	uint32 sensor_range;
	uint32 mass;
	uint32 max_fuel;
	int max_fore_velocity;
	int max_aft_velocity;
	float turn_rate;		// radians per second

	struct structShipClass *next;
	struct structShipClass *prev;

	unsigned nr_weapons;
	struct structWeapon *weapons;		// points to head of a double-linked list

	unsigned nr_polygons;
	struct structPolygon *polygons[1];	// realloc'd to hold more than one polygon
};


struct structNote {
	struct structNote *next;
	struct structNote *prev;
	unsigned length;
	char text[1];				// realloc'd to hold more than one character
};
struct structScript {
	struct structScript *next;
	struct structScript *prev;
	unsigned length;
	char code[1];				// realloc'd to hold more than one character
};	
struct structUser {
	uint32 id;
	char username[32];
	char password[32];
	struct structColor color1;
	struct structColor color2;
	uint32 flags;			// bit 1 clear if user does not exist in region
	struct structNote *notes;
	unsigned nr_scripts;
	struct structScript *scripts;	// scripts written by user
	struct structUser *next;
	struct structUser *prev;
	unsigned nr_allies;
	uint32 allies[1];		// realloc'd to hold more than one ally
};


struct structMapClass {
	uint32 id;
	char name[32];			// star, black hole, nebula, planet, asteroid, wormhole
	struct structMapClass *next;
	struct structMapClass *prev;
	unsigned nr_polygons;
	struct structPolygon *polygons[1];	// realloc'd to hold more than one polygon
};


struct structMapObject {
	uint32 id;
	char name[32];
	uint32 class_id;
	struct structColor color1;
	struct structColor color2;
	uint32 quantity_metal;	uint32 server_id;
	uint32 quantity_gas;	uint32 destination_id;
	// server_id is used for wormholes and is the server_id of the destination point
	// destination_id is the mapobject id of the other wormhole	
	int gravity;
	struct structMapObject *next;
	struct structMapObject *prev;
};


struct structEvent {
	struct structEvent *next;
	struct structEvent *prev;
	unsigned length;
	char event[1];			// realloc'd to hold more than one character
};
struct structObject {
	uint32 id;			// object ID
	char name[32];			// ship name
	uint32 class_id;		// ship class ID
	uint32 user_id;			// ship owner
	int velocity;			// ship speed
	float heading;			// ship course
	unsigned armor[16];		// ship armor strength
	unsigned sheilds;		// ship sheild strength
	unsigned fuel;			// ship remaining fuel
	unsigned sections;		// 1bit per section, if set then section is missing

	uint32 flags;
	uint32 x;
	uint32 y;
	uint32 distance;		// distance object has to go for current move
	struct structObject *next;
	struct structObject *prev;

	unsigned nr_events;
	struct structEvent *events;
	struct structThread *thread;
	char order[1];			// realloc'd to hold more than one character
};


extern struct structShipClass *shipclass_db;
extern struct structUser *user_db;
extern struct structObject *object_db[16];
extern struct structMapClass *mapclass_db;
extern struct structMapObject *mapobject_db[16];
extern struct structScript *script_db;			// global scripts

extern uint32 nr_shipclass, nr_users, nr_objects[16], nr_mapclass, nr_mapobjects[16], nr_scripts;

extern uint32 max_user_id, max_object_id, total_objects, total_mapobjects;

extern char db_path[100];

////////////////////////////////////////////////

void		userLoadAll();
void		userSaveAll();
void		userSendAll(int s, uint32 uid, int resumed);
uint32		userValidate(char *uname, char *passwd);
uint32		userAdd(char *uname, char *passwd);
void		userStoreNote(uint32 uid, char *note);

void		mapLoadAll();
void		mapSaveAll();
void		mapSendAll(int s);

void		scriptLoadAll();

void		objectLoadAll();
void		objectSaveAll();
struct structObject *objectCreate(uint32 class);
void		objectDestroy(struct structObject *object);
void		objectSendAll(int s, uint32 uid);

void		shipclassLoadAll();
void		shipclassSendAll(int s);

#endif
