#include "net.h"

//////////////////////////////////////////////

struct client_connection *clients;
int nr_clients;
pthread_mutex_t clients_mutex;
unsigned client_gfx;

//////////////////////////////////////////////

int clientSend(int s, char *data){
	// returns -1 if failed

#ifdef __DEBUG_SERVER_NETWORK__
	printf(":\tserver -> client: %s\n", data);
#endif
	if(send(s, data, strlen(data)+1, 0) == -1){
		printf(":\tlost connection to client\n");
		return(-1);
	}
	return(1);
}


int clientRecv(int s, char *data){
	// returns -1 if failed
	int count; char c[2];

	count=0;
	do{
		if(recv(s, c, 1, 0) < 0){
			printf(":\tlost connection to client\n");
			return(-1);
		}
		if(c[0]<32 && c[0]>0){
			printf(":\tlost connection to client\n");
			return(-1);
		}
		data[count]=c[0];
		count++;
	}while(c[0]!='\0' && count<BUFFER_SIZE);
	if(count>1){
#ifdef __DEBUG_SERVER_NETWORK__
		printf(":\tclient -> server: (%d) %s\n", count, data);
#endif
		return(count);
	}
	printf(":\tgot crap from client\n");
	return(-1);
}

void clientListen(int port){
	char data[BUFFER_SIZE];
	struct client_connection *client, *tmp;
	struct sockaddr_in local, *nptr;
	struct sockaddr client_sockaddr;
	int sc, s, adrlen, count, c, n, max_fd, uid, resume;
	fd_set set;

	if((s = socket(AF_INET, SOCK_STREAM, IPPROTO_IP)) < 0){
		printf("SERVER ERROR: could not allocate socket\n\n");
		exit(EXIT_FAILURE);
	}

	// fcntl(s, F_SETFL, O_NONBLOCK);
	memset(&local, 0, sizeof(struct sockaddr_in));

	local.sin_family=AF_INET;
	local.sin_port=htons(port);
	local.sin_addr.s_addr=htonl(inet_addr("0.0.0.0"));
	bzero(&(local.sin_zero), 8);

	if(bind(s, (struct sockaddr *)&local, sizeof(local)) < 0){
		printf("SERVER ERROR: could not bind to socket\n\n");
		exit(EXIT_FAILURE);
	}

	nptr = (struct sockaddr_in *)&client_sockaddr;
	nr_clients = 0;
	clients = NULL;
	pthread_mutex_init(&clients_mutex, NULL);

	if(listen(s, 5) < 0){
		printf("SERVER ERROR: could not listen on socket\n\n");
		exit(EXIT_FAILURE);
	}

	printf(":\tserver active, awaiting connections...\n");

	do{
		max_fd = 0;
		if(nr_clients){
			FD_ZERO(&set);
			pthread_mutex_lock(&clients_mutex);
			client = clients;
			for(c=0; c<nr_clients; c++){
				if(max_fd < client->s) max_fd = client->s;
				FD_SET(client->s, &set);
				client = client->next;
			}
			pthread_mutex_unlock(&clients_mutex);
		}
		FD_SET(0, &set);	// stdin
		FD_SET(s, &set);
		if(max_fd < s) max_fd = s;
		n = select(max_fd + 1, &set, NULL, NULL, NULL);
		if(n && nr_clients){
			pthread_mutex_lock(&clients_mutex);
			client = clients;
			for(c=0; c<nr_clients; c++){
				if(FD_ISSET(client->s, &set)){
					count = clientRecv(client->s, data);
					if(count != -1){
						client = clientProcess(data, client);
					} else {
						client = clientDisconnect(client);
					}
				}
				client = client->next;
			}
			pthread_mutex_unlock(&clients_mutex);
		}
		if(FD_ISSET(0, &set)) serverShutdown();		// stdin
		if(FD_ISSET(s, &set)){
		  adrlen = sizeof(client_sockaddr);
		  if((sc = accept(s, (struct sockaddr *)&client_sockaddr, &adrlen)) != -1){
			if((uid = clientLogin(sc, &client_sockaddr, &adrlen, nptr, &resume)) > 0){
				if((tmp = (struct client_connection *)malloc(
				   sizeof(struct client_connection))) == NULL){
					data[0] = CMD__DISCONNECT; data[1] = '\0';
					clientSend(sc, data);
					close(sc);
					printf("LISTEN: couldn't allocate structure, client terminated\n\n");
				} else {
					pthread_mutex_lock(&clients_mutex);
					if(clients == NULL){
						tmp->next = tmp;
						tmp->prev = tmp;
					} else {
						tmp->next = clients;
						tmp->prev = clients->prev;
						clients->prev->next = tmp;
						clients->prev = tmp;
					}
					tmp->uid = uid;
					tmp->graphics = client_gfx;
					clients = tmp;
					tmp->s = sc;
					nr_clients++;
					pthread_mutex_unlock(&clients_mutex);
					printf("LISTEN: %d client(s) connected, new client (uid=%d)\n",
						nr_clients, uid);

					userSendAll(tmp->s, uid, resume);
					objectSendAll(tmp->s, uid);
					shipclassSendAll(tmp->s);
				}
			} else{
				close(sc);
				printf("LISTEN: invalid login, client terminated\n\n");
			}
		  }
		}
	}while(1);
}



int clientLogin(int sc, struct sockaddr *client, int *adrlen, struct sockaddr_in *nptr, int *resume){
        int count, x, rc, type;
	char uname[32], passwd[32];
	char data[BUFFER_SIZE];

        if((rc = getpeername(sc, client, adrlen)) < 0){
                printf("SERVER:   getpeername() failed.\n");
                return(0);
        }
        printf("LISTEN: connect from: %s:%d\n", inet_ntoa(nptr->sin_addr), nptr->sin_port);

	count = clientRecv(sc, data);
	if(count != -1){
		type = (int)data[0];
		sscanf(&data[1], "%s %s %d", uname, passwd, &client_gfx);

		if(strlen(uname)>0 && strlen(passwd)>2){
			if(type == CMD_RESUME){
				if((x = userValidate(uname, passwd)) > 0){
					sprintf(data, "%c%Ld %d %d %d %d", (char)CMD__CONNECTED, vm_time,
						nr_users, 1, nr_users,
						vm_time_resolution*vm_time_compression*1000);
					clientSend(sc, data);
					*resume = 1;
					return(x);
				}
			} else if(type == CMD_NEW){
				if((x = userAdd(uname, passwd)) > 0){
					sprintf(data, "%c%Ld %d %d %d %d", (char)CMD__CONNECTED, vm_time,
						nr_users, 1, nr_users,
						vm_time_resolution*vm_time_compression*1000);
					clientSend(sc, data);
					*resume = 0;
					return(x);
				}
			}
		}
	}
	data[0] = (char)CMD__FAIL; data[1] = '\0';
	clientSend(sc, data);
	return(0);
}


void (*commands[])() = {	cmdMessage, cmdUserLevel, cmdRebuild, cmdStatus, cmdScript,
				cmdOrder };
struct client_connection *clientProcess(char *data, struct client_connection *client){
	int command;

	client->last_time = time(NULL);

	command = (int)(*data);
	data++;

	if( (command < CMD_MSG) || (command > CMD_server_commands) ){
		client = clientDisconnect(client);
	} else (*commands[command-CMD_MSG])(data, client);

	return(client);
}


struct client_connection *clientDisconnect(struct client_connection *client){
	struct client_connection *tmp;
	unsigned uid;

	uid = client->uid;
	tmp = client->prev;
	tmp->next = client->next;
	client->next->prev = tmp;
	nr_clients--;
	close(client->s);
	if(client==clients) clients = client->next;
	free(client);
	client = tmp;
	printf("LISTEN: client (uid=%d) terminated\n", uid);
	return(client);
}


struct client_connection *clientIsOnline(unsigned uid){
	struct client_connection *tmp;
	int x;

	tmp = clients;
	for(x=0; x<nr_clients; x++){
		if(tmp->uid == uid) return(tmp);
		tmp = tmp->next;
	}
	return(NULL);
}

